{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "b21bb42f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:20.654806Z",
     "iopub.status.busy": "2024-08-12T10:00:20.654360Z",
     "iopub.status.idle": "2024-08-12T10:00:20.988443Z",
     "shell.execute_reply": "2024-08-12T10:00:20.987909Z"
    }
   },
   "outputs": [],
   "source": [
    "import sympy as sm\n",
    "import sympy.physics.mechanics as me\n",
    "sm.init_printing(use_latex='mathjax')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "71130014",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:20.990656Z",
     "iopub.status.busy": "2024-08-12T10:00:20.990433Z",
     "iopub.status.idle": "2024-08-12T10:00:20.994189Z",
     "shell.execute_reply": "2024-08-12T10:00:20.993774Z"
    }
   },
   "outputs": [],
   "source": [
    "class ReferenceFrame(me.ReferenceFrame):\n",
    "\n",
    "    def __init__(self, *args, **kwargs):\n",
    "\n",
    "        kwargs.pop('latexs', None)\n",
    "\n",
    "        lab = args[0].lower()\n",
    "        tex = r'\\hat{{{}}}_{}'\n",
    "\n",
    "        super(ReferenceFrame, self).__init__(*args,\n",
    "                                             latexs=(tex.format(lab, 'x'),\n",
    "                                                     tex.format(lab, 'y'),\n",
    "                                                     tex.format(lab, 'z')),\n",
    "                                             **kwargs)\n",
    "me.ReferenceFrame = ReferenceFrame"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "2a0ba10a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:20.995831Z",
     "iopub.status.busy": "2024-08-12T10:00:20.995678Z",
     "iopub.status.idle": "2024-08-12T10:00:21.009244Z",
     "shell.execute_reply": "2024-08-12T10:00:21.008833Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{a}_x + b\\hat{a}_y + c\\hat{b}_x + d\\hat{b}_y + e\\hat{c}_x + f\\hat{c}_y$"
      ],
      "text/plain": [
       "a a_x + b a_y + c b_x + d b_y + e c_x + f c_y"
      ]
     },
     "execution_count": 3,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "alpha, beta = sm.symbols('alpha, beta')\n",
    "a, b, c, d, e, f = sm.symbols('a, b, c, d, e, f')\n",
    "\n",
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "C = me.ReferenceFrame('C')\n",
    "\n",
    "B.orient_axis(A, alpha, A.x)\n",
    "C.orient_axis(B, beta, B.y)\n",
    "\n",
    "v = a*A.x + b*A.y + c*B.x + d*B.y + e*C.x + f*C.y\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "b760af35",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.011336Z",
     "iopub.status.busy": "2024-08-12T10:00:21.010995Z",
     "iopub.status.idle": "2024-08-12T10:00:21.018335Z",
     "shell.execute_reply": "2024-08-12T10:00:21.017835Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle 0$"
      ],
      "text/plain": [
       "0"
      ]
     },
     "execution_count": 4,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAx = v.dot(A.x).diff(alpha)\n",
    "dvdalphaAx"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "cac75f09",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.020032Z",
     "iopub.status.busy": "2024-08-12T10:00:21.019874Z",
     "iopub.status.idle": "2024-08-12T10:00:21.030590Z",
     "shell.execute_reply": "2024-08-12T10:00:21.030196Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle - d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)}$"
      ],
      "text/plain": [
       "-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAy = v.dot(A.y).diff(alpha)\n",
    "dvdalphaAy"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "48bc1fde",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.032325Z",
     "iopub.status.busy": "2024-08-12T10:00:21.032164Z",
     "iopub.status.idle": "2024-08-12T10:00:21.042015Z",
     "shell.execute_reply": "2024-08-12T10:00:21.041596Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)}$"
      ],
      "text/plain": [
       "d⋅cos(α) + e⋅sin(α)⋅sin(β) + f⋅cos(α)"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAz = v.dot(A.z).diff(alpha)\n",
    "dvdalphaAz"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "85bed9a9",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.043808Z",
     "iopub.status.busy": "2024-08-12T10:00:21.043649Z",
     "iopub.status.idle": "2024-08-12T10:00:21.049862Z",
     "shell.execute_reply": "2024-08-12T10:00:21.049496Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (- d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)})\\hat{a}_y + (d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)})\\hat{a}_z$"
      ],
      "text/plain": [
       "(-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)) a_y + (d⋅cos(α) + e⋅sin(α)⋅sin(β) + f\n",
       "⋅cos(α)) a_z"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaA = dvdalphaAx*A.x + dvdalphaAy*A.y + dvdalphaAz*A.z\n",
    "dvdalphaA"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "54275cf1",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.051512Z",
     "iopub.status.busy": "2024-08-12T10:00:21.051352Z",
     "iopub.status.idle": "2024-08-12T10:00:21.071045Z",
     "shell.execute_reply": "2024-08-12T10:00:21.070657Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (- d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)})\\hat{a}_y + (d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)})\\hat{a}_z$"
      ],
      "text/plain": [
       "(-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)) a_y + (d⋅cos(α) + e⋅sin(α)⋅sin(β) + f\n",
       "⋅cos(α)) a_z"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(alpha, A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "0ec1078f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.072642Z",
     "iopub.status.busy": "2024-08-12T10:00:21.072503Z",
     "iopub.status.idle": "2024-08-12T10:00:21.080531Z",
     "shell.execute_reply": "2024-08-12T10:00:21.080121Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\cos{\\left(\\beta \\right)}\\hat{b}_x -  \\sin{\\left(\\beta \\right)}\\hat{b}_z$"
      ],
      "text/plain": [
       "cos(β) b_x + -sin(β) b_z"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdeBx = v.dot(B.x).diff(e)\n",
    "dvdeBy = v.dot(B.y).diff(e)\n",
    "dvdeBz = v.dot(B.z).diff(e)\n",
    "dvdeBx*B.x + dvdeBy*B.y + dvdeBz*B.z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "69e84a21",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.082411Z",
     "iopub.status.busy": "2024-08-12T10:00:21.081942Z",
     "iopub.status.idle": "2024-08-12T10:00:21.086964Z",
     "shell.execute_reply": "2024-08-12T10:00:21.086510Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\cos{\\left(\\beta \\right)}\\hat{b}_x -  \\sin{\\left(\\beta \\right)}\\hat{b}_z$"
      ],
      "text/plain": [
       "cos(β) b_x + -sin(β) b_z"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(e, B).express(B)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "d2e344a4",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.088743Z",
     "iopub.status.busy": "2024-08-12T10:00:21.088494Z",
     "iopub.status.idle": "2024-08-12T10:00:21.092648Z",
     "shell.execute_reply": "2024-08-12T10:00:21.092159Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle q{\\left(t \\right)}$"
      ],
      "text/plain": [
       "q(t)"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "t = sm.symbols('t')\n",
    "q_of = sm.Function('q')\n",
    "\n",
    "q = q_of(t)\n",
    "q"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "30d910e8",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.094290Z",
     "iopub.status.busy": "2024-08-12T10:00:21.094121Z",
     "iopub.status.idle": "2024-08-12T10:00:21.098000Z",
     "shell.execute_reply": "2024-08-12T10:00:21.097513Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{d}{d t} q{\\left(t \\right)}$"
      ],
      "text/plain": [
       "d       \n",
       "──(q(t))\n",
       "dt      "
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q.diff(t)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "0d1d2fec",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.099698Z",
     "iopub.status.busy": "2024-08-12T10:00:21.099543Z",
     "iopub.status.idle": "2024-08-12T10:00:21.103902Z",
     "shell.execute_reply": "2024-08-12T10:00:21.103503Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left( q_{1}{\\left(t \\right)}, \\  q_{2}{\\left(t \\right)}, \\  q_{3}{\\left(t \\right)}\\right)$"
      ],
      "text/plain": [
       "(q₁(t), q₂(t), q₃(t))"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q1, q2, q3 = me.dynamicsymbols('q1, q2, q3')\n",
    "q1, q2, q3"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "1b460a80",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.105442Z",
     "iopub.status.busy": "2024-08-12T10:00:21.105293Z",
     "iopub.status.idle": "2024-08-12T10:00:21.107514Z",
     "shell.execute_reply": "2024-08-12T10:00:21.107128Z"
    }
   },
   "outputs": [],
   "source": [
    "t = me.dynamicsymbols._t"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "0095e37e",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.109078Z",
     "iopub.status.busy": "2024-08-12T10:00:21.108930Z",
     "iopub.status.idle": "2024-08-12T10:00:21.115185Z",
     "shell.execute_reply": "2024-08-12T10:00:21.114786Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left( \\dot{q}_{1}, \\  \\ddot{q}_{2}, \\  \\dddot{q}_{3}\\right)$"
      ],
      "text/plain": [
       "(q₁̇, q₂̈, q₃⃛)"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "me.init_vprinting(use_latex='mathjax')\n",
    "q1.diff(t), q2.diff(t, 2), q3.diff(t, 3)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "3f0f7ae3",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.116740Z",
     "iopub.status.busy": "2024-08-12T10:00:21.116586Z",
     "iopub.status.idle": "2024-08-12T10:00:21.132051Z",
     "shell.execute_reply": "2024-08-12T10:00:21.131668Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle q_{1}\\hat{a}_x + q_{2}\\hat{a}_y + t^{2}\\hat{a}_z$"
      ],
      "text/plain": [
       "                   2\n",
       "q₁ a_x + q₂ a_y + t  a_z"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "B.orient_body_fixed(A, (q1, q2, q3), 'ZXZ')\n",
    "v = q1*A.x + q2*A.y + t**2*A.z\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "34f3a9c0",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.133727Z",
     "iopub.status.busy": "2024-08-12T10:00:21.133572Z",
     "iopub.status.idle": "2024-08-12T10:00:21.140659Z",
     "shell.execute_reply": "2024-08-12T10:00:21.140265Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\dot{q}_{1}\\hat{a}_x + \\dot{q}_{2}\\hat{a}_y + 2 t\\hat{a}_z$"
      ],
      "text/plain": [
       "q₁̇ a_x + q₂̇ a_y + 2⋅t a_z"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(t, A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "id": "694d877d",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:21.142257Z",
     "iopub.status.busy": "2024-08-12T10:00:21.142089Z",
     "iopub.status.idle": "2024-08-12T10:00:21.147633Z",
     "shell.execute_reply": "2024-08-12T10:00:21.147201Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\dot{q}_{1}\\hat{a}_x + \\dot{q}_{2}\\hat{a}_y + 2 t\\hat{a}_z$"
      ],
      "text/plain": [
       "q₁̇ a_x + q₂̇ a_y + 2⋅t a_z"
      ]
     },
     "execution_count": 18,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.dt(A)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.11.9"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}