{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "6066979c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:11.854033Z",
     "iopub.status.busy": "2024-05-02T10:06:11.853861Z",
     "iopub.status.idle": "2024-05-02T10:06:12.235118Z",
     "shell.execute_reply": "2024-05-02T10:06:12.234536Z"
    }
   },
   "outputs": [],
   "source": [
    "import sympy as sm\n",
    "import sympy.physics.mechanics as me\n",
    "sm.init_printing(use_latex='mathjax')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "535dfdaf",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.237781Z",
     "iopub.status.busy": "2024-05-02T10:06:12.237332Z",
     "iopub.status.idle": "2024-05-02T10:06:12.241340Z",
     "shell.execute_reply": "2024-05-02T10:06:12.240864Z"
    }
   },
   "outputs": [],
   "source": [
    "class ReferenceFrame(me.ReferenceFrame):\n",
    "\n",
    "    def __init__(self, *args, **kwargs):\n",
    "\n",
    "        kwargs.pop('latexs', None)\n",
    "\n",
    "        lab = args[0].lower()\n",
    "        tex = r'\\hat{{{}}}_{}'\n",
    "\n",
    "        super(ReferenceFrame, self).__init__(*args,\n",
    "                                             latexs=(tex.format(lab, 'x'),\n",
    "                                                     tex.format(lab, 'y'),\n",
    "                                                     tex.format(lab, 'z')),\n",
    "                                             **kwargs)\n",
    "me.ReferenceFrame = ReferenceFrame"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "7bba0327",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.243224Z",
     "iopub.status.busy": "2024-05-02T10:06:12.243046Z",
     "iopub.status.idle": "2024-05-02T10:06:12.257594Z",
     "shell.execute_reply": "2024-05-02T10:06:12.256997Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle a\\hat{a}_x + b\\hat{a}_y + c\\hat{b}_x + d\\hat{b}_y + e\\hat{c}_x + f\\hat{c}_y$"
      ],
      "text/plain": [
       "a a_x + b a_y + c b_x + d b_y + e c_x + f c_y"
      ]
     },
     "execution_count": 3,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "alpha, beta = sm.symbols('alpha, beta')\n",
    "a, b, c, d, e, f = sm.symbols('a, b, c, d, e, f')\n",
    "\n",
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "C = me.ReferenceFrame('C')\n",
    "\n",
    "B.orient_axis(A, alpha, A.x)\n",
    "C.orient_axis(B, beta, B.y)\n",
    "\n",
    "v = a*A.x + b*A.y + c*B.x + d*B.y + e*C.x + f*C.y\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "9791a9bb",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.260002Z",
     "iopub.status.busy": "2024-05-02T10:06:12.259625Z",
     "iopub.status.idle": "2024-05-02T10:06:12.267584Z",
     "shell.execute_reply": "2024-05-02T10:06:12.267119Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle 0$"
      ],
      "text/plain": [
       "0"
      ]
     },
     "execution_count": 4,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAx = v.dot(A.x).diff(alpha)\n",
    "dvdalphaAx"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "4370915f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.269810Z",
     "iopub.status.busy": "2024-05-02T10:06:12.269392Z",
     "iopub.status.idle": "2024-05-02T10:06:12.281738Z",
     "shell.execute_reply": "2024-05-02T10:06:12.281227Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle - d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)}$"
      ],
      "text/plain": [
       "-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAy = v.dot(A.y).diff(alpha)\n",
    "dvdalphaAy"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "71f6c35a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.283678Z",
     "iopub.status.busy": "2024-05-02T10:06:12.283505Z",
     "iopub.status.idle": "2024-05-02T10:06:12.293921Z",
     "shell.execute_reply": "2024-05-02T10:06:12.293430Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)}$"
      ],
      "text/plain": [
       "d⋅cos(α) + e⋅sin(α)⋅sin(β) + f⋅cos(α)"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaAz = v.dot(A.z).diff(alpha)\n",
    "dvdalphaAz"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "b55b749f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.295912Z",
     "iopub.status.busy": "2024-05-02T10:06:12.295707Z",
     "iopub.status.idle": "2024-05-02T10:06:12.303288Z",
     "shell.execute_reply": "2024-05-02T10:06:12.302788Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (- d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)})\\hat{a}_y + (d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)})\\hat{a}_z$"
      ],
      "text/plain": [
       "(-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)) a_y + (d⋅cos(α) + e⋅sin(α)⋅sin(β) + f\n",
       "⋅cos(α)) a_z"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdalphaA = dvdalphaAx*A.x + dvdalphaAy*A.y + dvdalphaAz*A.z\n",
    "dvdalphaA"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "507109e8",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.305247Z",
     "iopub.status.busy": "2024-05-02T10:06:12.305078Z",
     "iopub.status.idle": "2024-05-02T10:06:12.329088Z",
     "shell.execute_reply": "2024-05-02T10:06:12.328493Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle (- d \\sin{\\left(\\alpha \\right)} + e \\sin{\\left(\\beta \\right)} \\cos{\\left(\\alpha \\right)} - f \\sin{\\left(\\alpha \\right)})\\hat{a}_y + (d \\cos{\\left(\\alpha \\right)} + e \\sin{\\left(\\alpha \\right)} \\sin{\\left(\\beta \\right)} + f \\cos{\\left(\\alpha \\right)})\\hat{a}_z$"
      ],
      "text/plain": [
       "(-d⋅sin(α) + e⋅sin(β)⋅cos(α) - f⋅sin(α)) a_y + (d⋅cos(α) + e⋅sin(α)⋅sin(β) + f\n",
       "⋅cos(α)) a_z"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(alpha, A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "a7cf9f8d",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.331452Z",
     "iopub.status.busy": "2024-05-02T10:06:12.331076Z",
     "iopub.status.idle": "2024-05-02T10:06:12.340834Z",
     "shell.execute_reply": "2024-05-02T10:06:12.340233Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\cos{\\left(\\beta \\right)}\\hat{b}_x -  \\sin{\\left(\\beta \\right)}\\hat{b}_z$"
      ],
      "text/plain": [
       "cos(β) b_x + -sin(β) b_z"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "dvdeBx = v.dot(B.x).diff(e)\n",
    "dvdeBy = v.dot(B.y).diff(e)\n",
    "dvdeBz = v.dot(B.z).diff(e)\n",
    "dvdeBx*B.x + dvdeBy*B.y + dvdeBz*B.z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "ada4907d",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.343075Z",
     "iopub.status.busy": "2024-05-02T10:06:12.342658Z",
     "iopub.status.idle": "2024-05-02T10:06:12.348625Z",
     "shell.execute_reply": "2024-05-02T10:06:12.348019Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\cos{\\left(\\beta \\right)}\\hat{b}_x -  \\sin{\\left(\\beta \\right)}\\hat{b}_z$"
      ],
      "text/plain": [
       "cos(β) b_x + -sin(β) b_z"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(e, B).express(B)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "726cb222",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.350634Z",
     "iopub.status.busy": "2024-05-02T10:06:12.350454Z",
     "iopub.status.idle": "2024-05-02T10:06:12.355278Z",
     "shell.execute_reply": "2024-05-02T10:06:12.354687Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle q{\\left(t \\right)}$"
      ],
      "text/plain": [
       "q(t)"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "t = sm.symbols('t')\n",
    "q_of = sm.Function('q')\n",
    "\n",
    "q = q_of(t)\n",
    "q"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "3be8e30a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.357577Z",
     "iopub.status.busy": "2024-05-02T10:06:12.357215Z",
     "iopub.status.idle": "2024-05-02T10:06:12.361989Z",
     "shell.execute_reply": "2024-05-02T10:06:12.361512Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\frac{d}{d t} q{\\left(t \\right)}$"
      ],
      "text/plain": [
       "d       \n",
       "──(q(t))\n",
       "dt      "
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q.diff(t)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "2708b156",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.364181Z",
     "iopub.status.busy": "2024-05-02T10:06:12.363744Z",
     "iopub.status.idle": "2024-05-02T10:06:12.368703Z",
     "shell.execute_reply": "2024-05-02T10:06:12.368234Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left( q_{1}{\\left(t \\right)}, \\  q_{2}{\\left(t \\right)}, \\  q_{3}{\\left(t \\right)}\\right)$"
      ],
      "text/plain": [
       "(q₁(t), q₂(t), q₃(t))"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q1, q2, q3 = me.dynamicsymbols('q1, q2, q3')\n",
    "q1, q2, q3"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "5a2eaee9",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.370536Z",
     "iopub.status.busy": "2024-05-02T10:06:12.370366Z",
     "iopub.status.idle": "2024-05-02T10:06:12.372910Z",
     "shell.execute_reply": "2024-05-02T10:06:12.372447Z"
    }
   },
   "outputs": [],
   "source": [
    "t = me.dynamicsymbols._t"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "0d7046da",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.374652Z",
     "iopub.status.busy": "2024-05-02T10:06:12.374483Z",
     "iopub.status.idle": "2024-05-02T10:06:12.381446Z",
     "shell.execute_reply": "2024-05-02T10:06:12.380837Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left( \\dot{q}_{1}, \\  \\ddot{q}_{2}, \\  \\dddot{q}_{3}\\right)$"
      ],
      "text/plain": [
       "(q₁̇, q₂̈, q₃⃛)"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "me.init_vprinting(use_latex='mathjax')\n",
    "q1.diff(t), q2.diff(t, 2), q3.diff(t, 3)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "cef667ae",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.383431Z",
     "iopub.status.busy": "2024-05-02T10:06:12.383260Z",
     "iopub.status.idle": "2024-05-02T10:06:12.760442Z",
     "shell.execute_reply": "2024-05-02T10:06:12.759791Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle q_{1}\\hat{a}_x + q_{2}\\hat{a}_y + t^{2}\\hat{a}_z$"
      ],
      "text/plain": [
       "                   2\n",
       "q₁ a_x + q₂ a_y + t  a_z"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "B.orient_body_fixed(A, (q1, q2, q3), 'ZXZ')\n",
    "v = q1*A.x + q2*A.y + t**2*A.z\n",
    "v"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "d84f25a2",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.762883Z",
     "iopub.status.busy": "2024-05-02T10:06:12.762497Z",
     "iopub.status.idle": "2024-05-02T10:06:12.770423Z",
     "shell.execute_reply": "2024-05-02T10:06:12.769957Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\dot{q}_{1}\\hat{a}_x + \\dot{q}_{2}\\hat{a}_y + 2 t\\hat{a}_z$"
      ],
      "text/plain": [
       "q₁̇ a_x + q₂̇ a_y + 2⋅t a_z"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.diff(t, A)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "id": "cad28823",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-05-02T10:06:12.772445Z",
     "iopub.status.busy": "2024-05-02T10:06:12.772109Z",
     "iopub.status.idle": "2024-05-02T10:06:12.778274Z",
     "shell.execute_reply": "2024-05-02T10:06:12.777707Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\dot{q}_{1}\\hat{a}_x + \\dot{q}_{2}\\hat{a}_y + 2 t\\hat{a}_z$"
      ],
      "text/plain": [
       "q₁̇ a_x + q₂̇ a_y + 2⋅t a_z"
      ]
     },
     "execution_count": 18,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "v.dt(A)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.14"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}